package interpreter;

// Ação de alternativa do workflow
public class AlternationAction extends ConditionalAction {

    // Ação realizada caso a condição seja falsa (se for null, nenhuma ação é executada)
    private Action falseAction=null;

    public AlternationAction(Action action, BooleanExpression condition, Action falseAction) {
        super(action, condition);
        this.falseAction = falseAction;
    }

    public AlternationAction(Action action, BooleanExpression condition) {
        super(action, condition);
    }

    // Implementação da operação interpret
    // Avalia a condição e executa a operação para true ou para false
    public void interpret(Object context) {
        if(this.getCondition().evaluate(context))
            this.getAction().interpret(context);
        else{
            if(falseAction != null) this.falseAction.interpret(context);
        }
    }

    // Implementação da geração do XML
    // Gera o XML para a condição, para a ação true e para a ação false
    public String generateXML() {
        String returnXML = "<AlternationAction>";

        String conditionName = this.getCondition().getClass().getSimpleName();

        returnXML = returnXML.concat(
                "<" + conditionName + ">" +
                    this.getCondition().getParameter().toString() +
                "</" + conditionName + ">" +
                        this.getAction().generateXML());

        if(this.falseAction != null) {
            returnXML = returnXML.concat(
                    "<Else>" + this.falseAction.generateXML() + "</Else>");
        }

        return returnXML.concat("</AlternationAction>");
    }

}
