package interpreter;

import interpreter.commands.AddTemperatureCommand;
import interpreter.commands.AddWaterLevelCommand;
import interpreter.commands.SetPumperOnCommand;
import interpreter.contexts.WaterPumperAgent;
import interpreter.expressions.MaxTemperatureExpression;
import interpreter.expressions.MaxWaterLevelExpression;
import interpreter.expressions.PumperOnExpression;

public class Client {

    public static void main(String[] args) {

        // Retorna uma ação inicial de um workflow
        Action workflow = defineWorkflow();

        // Instancia o context (objeto alvo do workflow)
        WaterPumperAgent agent = new WaterPumperAgent(false, 0, 20);

        // Exibe as informações do context antes da interpretação
        System.out.println("Agent status:\nPumper on: " + agent.isPumperOn() +
                            "\nWater Level: " + agent.getWaterLevel() +
                            "\nTemperature: " + agent.getTemperature() + "\n");

        // Execução da operação interpret do workflow
        workflow.interpret(agent);

        // Exibe as informações do context após a interpretação
        System.out.println("Agent status:\nPumper on: " + agent.isPumperOn() +
                "\nWater Level: " + agent.getWaterLevel() +
                "\nTemperature: " + agent.getTemperature() + "\n");

        // Exibe o workflow em formato XML
        System.out.println(workflow.generateXML());
    }

    public static Action defineWorkflow() {

        // Inicio do workflow
        // Se a bomba estiver desligada, ligar bomba. Se não, fim do workflow.
        // Enquanto o nivel da agua estiver abaixo de 10, aumentar o nivel da agua em 1 e a temperatura em 5
        // Caso a temperatura seja superior a 50, desligar a bomba
        // Fim do workflow
        
        //commands
        SetPumperOnCommand pumperOnCommand = new SetPumperOnCommand(true);
        SetPumperOnCommand pumperOffCommand = new SetPumperOnCommand(false);
        AddWaterLevelCommand addWaterLevelCommand = new AddWaterLevelCommand(1);
        AddTemperatureCommand addTemperatureCommand = new AddTemperatureCommand(5);

        //expressions
        PumperOnExpression pumperNotOnExpression = new PumperOnExpression(false);
        MaxWaterLevelExpression maxWaterLevelExpression = new MaxWaterLevelExpression(10);
        MaxTemperatureExpression maxTemperatureExpression = new MaxTemperatureExpression(50);

        // Definição do Workflow
        // Elementos terminais
        CommandAction turnOnPumper = new CommandAction(pumperOnCommand);
        CommandAction turnOffPumper = new CommandAction(pumperOffCommand);
        CommandAction addToTemperature = new CommandAction(addTemperatureCommand);
        CommandAction addWaterLevel = new CommandAction(addWaterLevelCommand);

        // Elementos não terminais
        SequenceAction addWaterLevelThenAddTemperature = new SequenceAction(addWaterLevel, addToTemperature);
        AlternationAction verifyTemperature = new AlternationAction(turnOffPumper, maxTemperatureExpression);

        WhileRepetitionAction doUntilMaxWaterLevel = new WhileRepetitionAction(addWaterLevelThenAddTemperature, maxWaterLevelExpression);

        SequenceAction startPumperProcess = new SequenceAction(doUntilMaxWaterLevel, verifyTemperature);
        SequenceAction startWorkflow = new SequenceAction(turnOnPumper, startPumperProcess);

        return new AlternationAction(startWorkflow, pumperNotOnExpression);
    }


}

