package interpreter;

// Ação que define uma sequência de duas ações no workflow
public class SequenceAction implements Action {

    // Primeira ação que será executada
    private final Action firstAction;
    // Segunda ação que será executada
    private final Action secondAction;

    public SequenceAction(Action firstAction, Action secondAction) {
        this.firstAction = firstAction;
        this.secondAction = secondAction;
    }

    // Implementação da operação interpret que executa as duas ações em sequência
    public void interpret(Object context) {
        this.firstAction.interpret(context);
        this.secondAction.interpret(context);
    }

    // Implementação da operação generateXML que gerará o XML das duas ações
    public String generateXML() {

        String returnXML = "<SequenceAction>";

        returnXML = returnXML.concat(
                this.firstAction.generateXML() +
                this.secondAction.generateXML());

        return returnXML.concat("</SequenceAction>");
    }



}
